/*
 * endianness.h
 *
 * Endian-dependant encoding/decoding, header
 *
 * Copyright (c) 1998-2000, Virtual Unlimited B.V.
 *
 * Author: Bob Deblier <bob@virtualunlimited.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _ENDIANNESS_H
#define _ENDIANNESS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#ifdef __cplusplus
inline int16 swap16(int16 n)
{
	return (    ((n & 0xff) << 8) |
				((n & 0xff00) >> 8) );
}

inline uint16 swapu16(uint16 n)
{
	return (    ((n & 0xffU) << 8) |
				((n & 0xff00U) >> 8) );
}

inline int32 swap32(int32 n)
{
	#if (SIZEOF_LONG == 4)
	return (    ((n & 0xff) << 24) |
				((n & 0xff00) << 8) |
				((n & 0xff0000) >> 8) |
				((n & 0xff000000) >> 24) );
	#else
	return (    ((n & 0xffL) << 24) |
				((n & 0xff00L) << 8) |
				((n & 0xff0000L) >> 8) |
				((n & 0xff000000L) >> 24) );
	#endif
}

inline uint32 swapu32(uint32 n)
{
	#if (SIZEOF_UNSIGNED_LONG == 4)
	return (    ((n & 0xffU) << 24) |
				((n & 0xff00U) << 8) |
				((n & 0xff0000U) >> 8) |
				((n & 0xff000000U) >> 24) );
	#else
	return (    ((n & 0xffUL) << 24) |
				((n & 0xff00UL) << 8) |
				((n & 0xff0000UL) >> 8) |
				((n & 0xff000000UL) >> 24) );
	#endif
}

inline int64 swap64(int64 n)
{
	#if (SIZEOF_LONG == 4)
	return (    ((n & 0xffLL) << 56) |
				((n & 0xff00LL) << 40) |
				((n & 0xff0000LL) << 24) |
				((n & 0xff000000LL) << 8) |
				((n & 0xff00000000LL) >> 8) |
				((n & 0xff0000000000LL) >> 24) |
				((n & 0xff000000000000LL) >> 40) |
				((n & 0xff00000000000000LL) >> 56) );
	#else
	return (    ((n & 0xffL) << 56) |
				((n & 0xff00L) << 40) |
				((n & 0xff0000L) << 24) |
				((n & 0xff000000L) << 8) |
				((n & 0xff00000000L) >> 8) |
				((n & 0xff0000000000L) >> 24) |
				((n & 0xff000000000000L) >> 40) |
				((n & 0xff00000000000000L) >> 56) );
	#endif
}
#else
 int16 swap16 (int16);
uint16 swapu16(uint16);
 int32 swap32 (int32);
uint32 swapu32(uint32);
 int64 swap64 (int64);
#endif

#ifdef __cplusplus
extern "C" {
#endif

int encodeByte(javabyte, byte*);
int encodeShort(javashort, byte*);
int encodeInt(javaint, byte*);
int encodeLong(javalong, byte*);
int encodeChar(javachar, byte*);
int encodeFloat(javafloat, byte*);
int encodeDouble(javadouble, byte*);

int encodeInts(const javaint*, byte*, int);
int encodeChars(const javachar*, byte*, int);

int decodeByte(javabyte*, const byte*);
int decodeShort(javashort*, const byte*);
int decodeInt(javaint*, const byte*);
int decodeLong(javalong*, const byte*);
int decodeChar(javachar*, const byte*);
int decodeFloat(javafloat*, const byte*);
int decodeDouble(javadouble*, const byte*);

int decodeInts(javaint*, const byte*, int);
int decodeChars(javachar*, const byte*, int);

int writeByte(javabyte, FILE*);
int writeShort(javashort, FILE*);
int writeInt(javaint, FILE*);
int writeLong(javalong, FILE*);
int writeChar(javachar, FILE*);

int writeInts(const javaint*, FILE*, int);
int writeChars(const javachar*, FILE*, int);

int readByte(javabyte*, FILE*);
int readShort(javashort*, FILE*);
int readInt(javaint*, FILE*);
int readLong(javalong*, FILE*);
int readChar(javachar*, FILE*);

int readInts(javaint*, FILE*, int);
int readChars(javaint*, FILE*, int);

#ifdef __cplusplus
}
#endif

#endif
