/*
 * mtprng.h
 *
 * Mersenne twister pseudo-random number generator, header
 *
 * Copyright (c) 1998-2000, Virtual Unlimited B.V.
 *
 * Author: Bob Deblier <bob@virtualunlimited.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _MTPRNG_H
#define _MTPRNG_H

#if HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef _REENTRANT
#include "mutex.h"
#endif

#include "beecrypt.h"

#define N	624
#define M	397
#define K	0x9908B0DF

typedef struct
{
	#ifdef _REENTRANT
	MUTEX_T	lock;
	#endif
	uint32	state[N+1];
	uint32	left;
	uint32*	nextw;
} mtprngParam;

#ifdef __cplusplus
extern "C" {
#endif

extern const randomGenerator mtprng;

int mtprngSetup(mtprngParam*);
int mtprngSeed (mtprngParam*, const uint32*, int);
int mtprngNext (mtprngParam*, uint32*, int);

#ifdef __cplusplus
}
#endif

#endif
