/*
 * mutex.h
 *
 * mutex (MUTually EXclusive lock) header
 *
 * Copyright (c) 1999, Virtual Unlimited B.V.
 *
 * Author: Bob Deblier <bob@virtualunlimited.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef _MUTEX_LOCK_H
#define _MUTEX_LOCK_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#if HAVE_SYNCH_H
#include <synch.h>
#define MUTEX_T mutex_t
#define MUTEX_INIT(m) mutex_init(m, USYNC_THREAD, (void*) 0)
#define MUTEX_LOCK(m) mutex_lock(m)
#define MUTEX_UNLOCK(m) mutex_unlock(m)
#define MUTEX_DESTROY(m) mutex_destroy(m)
#elif HAVE_PTHREAD_H
#include <pthread.h>
#define MUTEX_T pthread_mutex_t
#define MUTEX_INIT(m) pthread_mutex_init(m, (pthread_mutexattr_t*) 0)
#define MUTEX_LOCK(m) pthread_mutex_lock(m)
#define MUTEX_UNLOCK(m) pthread_mutex_unlock(m)
#define MUTEX_DESTROY(m) pthread_mutex_destroy(m)
#else
#error need other mutex implementation
#endif

#endif
